# EPICS IOC Startup Script generated by https://github.com/epics-containers/ibek

cd "/epics/ioc"
dbLoadDatabase dbd/ioc.dbd
ioc_registerRecordDeviceDriver pdbbase

Command number 1 for subentity_test
Command number 2 for subentity_test
Command number 3 for subentity_test

dbLoadRecords /epics/runtime/ioc.db
iocInit
