# EPICS IOC Startup Script generated by https://github.com/epics-containers/ibek

cd "/epics/ioc"

epicsEnvSet Vec1 192
epicsEnvSet Vec2 193

dbLoadDatabase dbd/ioc.dbd
ioc_registerRecordDeviceDriver pdbbase

# ipacAddHy8002("slot, intLevel")
ipacAddHy8002("4, 2")
# record above carrier card ID 0 in 'IPAC4'
epicsEnvSet IPAC4 0
ipacAddHy8002("5, 2")
# record above carrier card ID 1 in 'IPAC5'
epicsEnvSet IPAC5 1

dbLoadRecords /epics/runtime/ioc.db
iocInit
